// Example of calling NetMaker from ROOT.
//
// NetMaker should be running with the project file pnn_root.NetPrj opened.
//
// In ROOT command line type:
//
// .L TNetMaker.cxx
// .L pNN.C
// TNtuple* ntuple_out = pNtuple();
//
//  or
//
// TNtuple* ntuple_out = pTree();
//


#include "TNetMaker.h"

TNtuple* pNtuple()
{
	// length of the training/testing vector
	// - includes all input values (event features) and target values
	Int_t length = 3;
	Float_t* v = new Float_t[length];

	// data format: 2 input values and single target value for each event
	TNtuple* net_input = new TNtuple("NetMaker", "Testing data", "d1:d2:t1");

	// Object that sends input data to NetMaker and receives processed outputs.
	// If NetMaker is running on a separate machine you need to change "localhost"
	// to the apropriate DNS name or IP number. Second parameter is the port number
	// on which NetMaker is listening. This value should match the number in the
	// "testing inp" block in the pnn_root.NetPrj project.
	//
	// The address listed in the "net output" block of the project should be set up
	// to the address of the computer with ROOT. Port number 20000 is default for
	// TNetMaker object and should not be changed in this example; if you need more
	// connections, use TNetMaker::SetLocalPort(Int_t port) method.
	TNetMaker* netmaker = new TNetMaker("localhost", 10001);

	Int_t ncols;
	FILE* fp = fopen("testing_data.prn", "r");
	if (fp) cout << "Reading input file." << endl;
	else { cout << "File not found." << endl; return NULL; }

	Int_t ii = 0;
	while (ii++ < 10)
	{
		ncols = fscanf(fp, "%f %f %f", v, v+1, v+2);
		if (ncols < 0) break;
		net_input->Fill(v);
	}
	fclose(fp);

	// *************** here is the communication part ***************
	cout << "Sending input data." << endl;
	Int_t nev = netmaker->Send(net_input);
	if (nev == net_input->GetEntries()) cout << nev << " events sent." << endl;
	else
	{
		cout << "We've got problem, no events sent - please, check the connection:" << endl;
		cout << " - NetMaker is running and the project file is open;" << endl;
		cout << " - host address, port numbers match;" << endl;
		cout << " - firewall is not blocking any port that we use." << endl << endl;
		cout << "Still not running - send me all the error messages." << endl << endl;
		delete net_input;
		delete netmaker;
		delete [] v;
		return NULL;
	}

	cout << "Please wait..." << endl;
	TNtuple* net_output = netmaker->GetNtuple();
	cout << "Processed data received (" << net_output->GetEntries() << " events)." << endl;
	// **************************************************************

	// ntuple contains:
	// 2 input values (branches i1 and i2)
	// 1 network output (o1)
	// 1 target value (t1)

	// some histogram...
	hout1 = new TH1F("out1", "net out1 distribution", 100, 0, 1);
	hout1->SetFillColor(48);

	Float_t out1;
	net_output->SetBranchAddress("o1", &out1);
	Int_t nsamples = net_output->GetEntries();
	for (Int_t i = 0; i < nsamples; i++)
	{
		net_output->GetEntry(i);
		hout1->Fill(out1);
	}
	TCanvas* c1 = new TCanvas("NetMaker", "output distribution", 600, 400);
	hout1->Draw();

	// *** caution: hout1 and c1 remain not deleted :)

	delete net_input;
	delete netmaker;
	delete [] v;

	// return ntuple received from NetMaker...
	return net_output;
}

TNtuple* pTree()
{
	Int_t length = 3;
	Float_t* v = new Float_t[length];

	TTree* net_input = new TTree("NetMaker", "Testing data");
	net_input->Branch("i1", v, "i1/F");
	net_input->Branch("i2", v+1, "i2/F");
	net_input->Branch("t1", v+2, "t1/F");

	Int_t ncols;
	FILE* fp = fopen("testing_data.prn", "r");
	if (fp) cout << "Reading input file." << endl;
	else { cout << "File not found." << endl; return NULL; }

	while (1)
	{
		ncols = fscanf(fp, "%f %f %f", v, v+1, v+2);
		if (ncols < 0) break;
		net_input->Fill();
	}
	fclose(fp);


	TNetMaker* netmaker = new TNetMaker("localhost", 10001);

	cout << "Sending input data." << endl;
	Int_t nev = netmaker->Send(net_input, "i1/F:i2/F:t1/F");
	if (nev == net_input->GetEntries()) cout << nev << " events sent." << endl;
	else
	{
		cout << nev << " events sent != " << net_input->GetEntries() << " entries" << endl;
		delete net_input;
		delete netmaker;
		delete [] v;
		return NULL;
	}

	cout << "Please wait..." << endl;
	TNtuple* net_output = netmaker->GetNtuple();
	cout << "Processed data received (" << net_output->GetEntries() << " events)." << endl;


	hout1 = new TH1F("out1", "net out1 distribution", 100, 0, 1);
	hout1->SetFillColor(48);

	Float_t out1;
	net_output->SetBranchAddress("o1", &out1);
	Int_t nsamples = net_output->GetEntries();
	for (Int_t i = 0; i < nsamples; i++)
	{
		net_output->GetEntry(i);
		hout1->Fill(out1);
	}
	TCanvas* c1 = new TCanvas("NetMaker", "output distribution", 600, 400);
	hout1->Draw();

	// *** caution: hout1 and c1 remain not deleted :)

	delete net_input;
	delete netmaker;
	delete [] v;

	return net_output;
}